#include "audio_device_manager.h"

#include "dummy_audio_device.h"
#include "port_audio_device.h"
#include "audio/mixer/mixer.h"

void audio_device_manager::init(singleton_initliazer& initliazer) {
    singleton_t<audio_device_manager>::init(initliazer);

    dummy_audio_device_ = new dummy_audio_device();
    main_audio_device_ = new port_audio_device();

    main_audio_device_->init();
    dummy_audio_device_->init();
}

void audio_device_manager::release() {
    singleton_t<audio_device_manager>::release();
    delete dummy_audio_device_;
    delete main_audio_device_;
}

void audio_device_manager::start() {
    main_audio_device_->start();
}

double audio_device_manager::get_sample_rate() const {
    return main_audio_device_->sample_rate();
}

void audio_device_manager::set_sample_rate(double sample_rate) {
    main_audio_device_->set_sample_rate(sample_rate);
    dummy_audio_device_->set_sample_rate(sample_rate);
}

uint32_t audio_device_manager::get_buffer_size() const {
    return main_audio_device_->buffer_size();
}

void audio_device_manager::set_buffer_size(int buffer_size) {
    main_audio_device_->set_buffer_size(buffer_size);
    dummy_audio_device_->set_buffer_size(buffer_size);
}

uint32_t audio_device_manager::get_input_channel_count() const {
    return 0;
}

uint32_t audio_device_manager::get_output_channel_count() const {
    return 2; // 现在是固定值, 以后可能会改
}
